#!/usr/bin/python

import os;
import re;

EXCLUDES = ['gtest'];


print "#build: ", os.uname();

# configure compiling parameters according to the operating system
if re.match("CYGWIN", os.uname()[0]):
  TESTLIB = "TESTLIB=../../gtest/gtest-all.o ../../gtest/gtest_main.o"
  CC = "CC=mingw32-g++"
  OPT = "OPT=-g -O2 -W -Wall -Werror -Wextra -mno-cygwin"
else:
  TESTLIB = "TESTLIB=-L ../../gtest -lgtest_main -lgtest -lpthread"
  CC = "CC=g++"
  OPT = "OPT=-g -O2 -m64 -W -Wall -Werror -Wextra"

# returns all includes from file (not system-ones)
def get_dependencies(filename, parent):
  dependency = [filename];

  m = re.match('(.*)/', filename)
  if m:
    dir = m.group(1)
  else: 
    dir = '.'

  try:
    for line in open(filename, 'r'):
      m = re.match('#include *"(.*)"', line);
      if not m:
        continue;

      ok = True
      for e in EXCLUDES:
        if re.match(e, m.group(1)):
          ok = False

      if not ok:
        continue;

      if re.search('/', m.group(1)):
        dependency += get_dependencies('../../' + m.group(1), filename)
      else:
        dependency += get_dependencies(dir + '/' + m.group(1), filename);
  except IOError as (errno, strerror):
    raise Exception('Problem locating dependency ' + filename +
    ' of ' + parent + "\n" + strerror);
  return list(set(dependency));

# returns name of the binaty given cpp file
def get_binary(filename):
  return re.sub('\.cpp', '.exe', filename);

# print compilation rule
def print_compile_rule(filename):
  deps = get_dependencies(filename, '__none__');
  bin = get_binary(filename);
  print "%s: " % bin,
  for dep in deps:
    print " %s" % dep,
  print
  print "\techo 'compiling %s'" % filename
  print "\t${CC} $I ${TESTLIB} ${OPT} %s -o %s" % (filename, bin)
  # generate assembly output
  #print "\t${CC} $I ${TESTLIB} ${OPT} %s -o %s.s -g -S -fverbose-asm" % (filename, filename)
  #print "\tas -alhnd %s.s > %s.lst" % (filename, filename)
  print

# print compile-test rule
def print_compiletest_rule(filename):
  deps = get_dependencies(filename, '__none__');
  bin = get_binary(filename);
  print ".PHONY: fail_%s" % bin
  print "fail_%s: " % bin,
  for dep in deps:
    print " %s" % dep,
  print
  print "\techo Testing compile-fail test %s" % filename
  print "\techo 'compiling %s' for pass" % filename
  print "\t${CC} $I ${OPT} %s -o %s -DCOMPILE_PASS" % (filename, bin)
  print "\t-rm '%s'" % bin
  print "\techo 'compiling %s' for fail" % filename
  print "\tif ${CC} $I ${OPT} %s -o %s -DCOMPILE_FAIL; then echo '***FAIL***'; exit 1; fi" % (filename, bin)
  print "\t-rm '%s'" % bin
  print "\techo 'Compile-fail test for %s succeeded'" % filename
  print "\techo '*****OK****'"
  print


#print beginning of the makefile
def print_headers():
  print TESTLIB
  print CC
  print "I=-I../.."
  print OPT
  print


# start of the script
all_files = os.listdir('.');
unittests = filter(lambda file : re.match('.*unittest.cpp$', file), all_files);
tests = filter(lambda file : re.match('.*_test.cpp$', file), all_files);
benchmarks = filter(lambda file : re.match('.*benchmark.cpp$', file), all_files);
compiletest = filter(lambda file : re.match('.*compiletest.cpp$', file), all_files);

print_headers();
compilable = unittests + benchmarks + tests;
for file in compilable:
  print_compile_rule(file);

print ".PHONY: compile"
print "compile: \\"
for filename in compilable:
  print "    %s \\" % get_binary(filename)
print
print

print ".PHONY: test"
print "test:",
for filename in unittests:
  print " %s" % get_binary(filename),
print
for filename in unittests:
  print "\t./%s" % get_binary(filename)
print

print ".PHONY: benchmark"
print "benchmark:",
for filename in benchmarks:
  print " %s" % get_binary(filename),
print
for filename in benchmarks:
  print "\t./%s" % get_binary(filename)
print

print ".PHONY: testall"
print "testall: test ",
for filename in tests:
  b = get_binary(filename)
  print "run_%s " % b,
print
print

for filename in tests:
  b = get_binary(filename);
  print ".PHONY: run_%s" % b
  print "run_%s: %s" % (b,b)
  print "\t./%s" % b
  print

print ".PHONY: compiletest"
print "compiletest: ",
for filename in compiletest:
  b = get_binary(filename)
  print "fail_%s" %b ,
print

for filename in compiletest:
  print_compiletest_rule(filename)

print
print ".PHONY: clean"
print "clean:"
print "\t-rm *.exe"

print
print ".PHONY: _makefile"
print "_makefile:"
print "\t../../automakefile.py > tmp && mv tmp Makefile"
