#include "gtest/gtest.h"
#include "skiplist.h"
#include "template/template.h"
#include "debug/ppdebug.h"
#include "utils/rand/rand.h"
#include <set>

namespace balanced_structures {
namespace skiplist {

TEST(SkiplistTest, compareToSet) {
  Skiplist<int> list(Rand(42));
  std::multiset<int> reference;
  Rand dataRand = Rand(47);

  const int MAX_VALUE = 1000000;
  const int CHECK_AFTER = 500000;
  const int CHECKS = 30;

  for (int i = 0; i < CHECKS; i++) {
    for (int j = 0; j < CHECK_AFTER; j++) {
      // insert new value
      int value = dataRand.rand(0, MAX_VALUE);
      list.insert(value);
      reference.insert(value);

      // delete some value
      value = dataRand.rand(0, MAX_VALUE);
      EXPECT_EQ(reference.find(value) == reference.end(),
                list.find(value) == list.end());
      if (reference.find(value) != reference.end()) {
        reference.erase(reference.find(value));
        list.erase(list.find(value));
      }
    }

    printf("Checking. List size: %d\n", (int) list.size());
    EXPECT_EQ(reference.size(), list.size());
    Skiplist<int>::iterator it_list = list.begin();
    std::multiset<int>::iterator it_reference = reference.begin();
    size_t pos = 0;
    while (it_reference != reference.end()) {
      EXPECT_EQ(*it_reference, *it_list);
      EXPECT_EQ(it_list, list.kth(pos));
      EXPECT_EQ(pos, list.xth(it_list));
      ++it_reference;
      ++it_list;
      pos++;
    }

    for (int i = -10; i < MAX_VALUE + 10; i++) {
      EXPECT_EQ(std::distance(reference.lower_bound(i),
                              reference.upper_bound(i)),
                std::distance(list.lower_bound(i),
                              list.upper_bound(i)));

    }
  }
}

} // namespace skiplist
} // namespace balanced_structures
