#ifndef H_BALANCED_STRUCTURES_SKIPLIST_SKIPLIST_UTILS
#define H_BALANCED_STRUCTURES_SKIPLIST_SKIPLIST_UTILS

namespace balanced_structures {
namespace skiplist {

namespace node_utils {
  /**
   * Generate level of node randomly with
   * geometric distribution with probablity \a p percent.
   * Note: The resulting distribution is geometric distribution with
   * cutoff at \a max_level (higher levels are accumulated to this last)
   *
   * @preconditions <i> 0 < p < 100 </i>.
   *
   * @param rand random number generator
   * @param levelup_prob_percent probability of <i> level+1 </i>  over <i> level </i>
   * @param max_level level cutoff
   *
   * @returns integer, level of a node, <i> 1 <= level <= max_level </i>
   */
  template <typename T>
  T randomLevel(Rand* rand, int levelup_prob_percent, T max_level) {
    Preconditions::check(1 < max_level);
    T level = 1;
    while ((level < max_level) && 
        (rand->rand(0, 100) < levelup_prob_percent)) {
      level++;
    }
    return level;
  }

} // namespace node_utils

} // namespace skiplist
} // namespace balanced_structures

#endif
