#include "gtest/gtest.h"
#include "angle.h"

namespace geometry {
namespace two_d {

TEST(Angle, quadrants)
{
  EXPECT_EQ(CENTER, getQuadrant(Point<int>(0, 0)));
  EXPECT_EQ(CENTER, getQuadrant(Point<double>(-0.0, -0.0)));

  EXPECT_EQ(TOP_RIGHT, getQuadrant(Point<int>(4, 0)));
  EXPECT_EQ(TOP_RIGHT, getQuadrant(Point<int>(4, 4)));

  EXPECT_EQ(TOP_LEFT, getQuadrant(Point<int>(0, 4)));
  EXPECT_EQ(TOP_LEFT, getQuadrant(Point<int>(-4, 4)));

  EXPECT_EQ(BOTTOM_LEFT, getQuadrant(Point<int>(-4, 0)));
  EXPECT_EQ(BOTTOM_LEFT, getQuadrant(Point<int>(-4, -4)));

  EXPECT_EQ(BOTTOM_RIGHT, getQuadrant(Point<int>(0, -4)));
  EXPECT_EQ(BOTTOM_RIGHT, getQuadrant(Point<int>(4, -4)));
}

TEST(Angle, lessIsFalseWithSameValues) {
  EXPECT_FALSE(angleLess(Point<int>(0, 0), Point<int>(0, 0)));
  EXPECT_FALSE(angleLess(Point<int>(1, 0), Point<int>(1, 0)));
  EXPECT_FALSE(angleLess(Point<int>(0, 1), Point<int>(0, 1)));
  EXPECT_FALSE(angleLess(Point<int>(1, 1), Point<int>(1, 1)));
  EXPECT_FALSE(angleLess(Point<int>(-1, 0), Point<int>(-1, 0)));
  EXPECT_FALSE(angleLess(Point<int>(0, -1), Point<int>(0, -1)));
  EXPECT_FALSE(angleLess(Point<int>(-1, -1), Point<int>(-1, -1)));
  EXPECT_FALSE(angleLess(Point<double>(0.0, -0.0), Point<double>(0.0, -0.0)));
}

TEST(Angle, lessZeroIsSmallest) {
  EXPECT_TRUE(angleLess(Point<int>(0, 0), Point<int>(1, 0)));
  EXPECT_TRUE(angleLess(Point<int>(0, 0), Point<int>(0, 1)));
  EXPECT_TRUE(angleLess(Point<int>(0, 0), Point<int>(1, 1)));
  EXPECT_TRUE(angleLess(Point<int>(0, 0), Point<int>(-1, 0)));
  EXPECT_TRUE(angleLess(Point<int>(0, 0), Point<int>(0, -1)));
  EXPECT_TRUE(angleLess(Point<int>(0, 0), Point<int>(-1, -1)));
}

TEST(Angle, lessWorksOk) {
  const int STEPS = 720;
  for (int i = 0; i < STEPS; i++) {
    for (int j = 0; j < STEPS; j++) {
      double r1 = 1e-6 + (i * 30 + j*20) % 47;
      double r2 = 1e-6 + (i * 20 + j*30) % 47;
      double a1 = 2 * M_PI * (double) i / STEPS;
      double a2 = 2 * M_PI * (double) j / STEPS;
      Point<double> p1(cos(a1) * r1, sin(a1) * r1);
      Point<double> p2(cos(a2) * r2, sin(a2) * r2);
      EXPECT_EQ(i < j, angleLess(p1, p2));
    }
  }
}
} // namespace two_d
} // namespace geometry

