#include "gtest/gtest.h"
#include "point.h"
#include "distance.h"
#include "math/rational/rational.h"

namespace geometry {
namespace two_d {

using namespace math::rational;
/**
 * Test square distance between points.
 */
TEST(Distance, sqrDistancePointPoint) {
  EXPECT_EQ(5, sqrDistancePointPoint(
        Point<int>(0,0), Point<int>(1,2)));

  EXPECT_EQ(5, sqrDistancePointPoint(
        Point<int>(0,0), Point<int>(-1,-2)));

  EXPECT_EQ(5, sqrDistancePointPoint(
        Point<int>(-1,0), Point<int>(0, 2)));

  EXPECT_EQ(765, sqrDistancePointPoint(
        Point<int>(22,37), Point<int>(43, 55)));
}

/**
 * Test distance between points.
 */
TEST(Distance, distancePointPoint) {
  EXPECT_NEAR(1, distancePointPoint(
        Point<int>(0,0), Point<int>(1,0)), 1e-10);

  EXPECT_NEAR(sqrt(765), distancePointPoint(
        Point<int>(12,27), Point<int>(33,45)), 1e-10);

  EXPECT_NEAR(sqrt(72), distancePointPoint(
        Point<int>(-3,-3), Point<int>(3,3)), 1e-10);
}

TEST(Distance, SimpleSqrDistancePointLine)
{
  Point<int> a(0,0);
  Point<int> b(0,10);
  Point<int> c(10,0);
  LineSegment<int> l1(a, b);
  LineSegment<int> l2(a, c);

  EXPECT_EQ(Rational<int>(0),
      sqrDistancePointLine(Point<int>(0, 2), l1));

  EXPECT_EQ(Rational<int>(0),
      sqrDistancePointLine(Point<int>(2, 0), l2));

  EXPECT_EQ(Rational<int>(4),
      sqrDistancePointLine(Point<int>(0, 2), l2));

  EXPECT_EQ(Rational<int>(100),
      sqrDistancePointLine(Point<int>(-10, 1000), l1));
}

TEST(Distance, SimpleDistancePointLine)
{
  Point<int> a(0,0);
  Point<int> b(0,10);
  Point<int> c(10,0);
  LineSegment<int> l1(a, b);
  LineSegment<int> l2(a, c);

  EXPECT_NEAR(0,
      distancePointLine(Point<int>(0, 2), l1), 1e-10);

  EXPECT_NEAR(0,
      distancePointLine(Point<int>(2, 0), l2), 1e-10);

  EXPECT_NEAR(2,
      distancePointLine(Point<int>(0, 2), l2), 1e-10);

  EXPECT_NEAR(10,
      distancePointLine(Point<int>(-10, 1000), l1), 1e-10);
}


TEST(Distance, SimpleSqrDistancePointLineSegment)
{

  Point<int> a(0,0);
  Point<int> b(0,10);
  LineSegment<int> l1(a, b);

  EXPECT_EQ(Rational<int>(0),
      sqrDistancePointLine(Point<int>(0, 2), l1));
  
  EXPECT_EQ(Rational<int>(4),
      sqrDistancePointLineSegment(Point<int>(0, -2), l1));

  EXPECT_EQ(Rational<int>(2),
      sqrDistancePointLineSegment(Point<int>(1, 11), l1));


  EXPECT_EQ(Rational<int>(5),
      sqrDistancePointLineSegment(Point<int>(2, -1), l1));

  EXPECT_EQ(Rational<int>(4),
      sqrDistancePointLineSegment(Point<int>(-2, 0), l1));

  EXPECT_EQ(Rational<int>(4),
      sqrDistancePointLineSegment(Point<int>(2, 6), l1));

  EXPECT_EQ(Rational<int>(4),
      sqrDistancePointLineSegment(Point<int>(2, 10), l1));

  EXPECT_EQ(Rational<int>(13),
      sqrDistancePointLineSegment(Point<int>(-2, 13), l1));
}

TEST(Distance, SimpleDistancePointLineSegment)
{
  Point<int> a(0,0);
  Point<int> b(0,10);
  LineSegment<int> l1(a, b);

  EXPECT_NEAR(0,
      distancePointLine(Point<int>(0, 2), l1), 1e-10);
  
  EXPECT_NEAR(2,
      distancePointLineSegment(Point<int>(0, -2), l1), 1e-10);

  EXPECT_NEAR(sqrt(2),
      distancePointLineSegment(Point<int>(1, 11), l1), 1e-10);


  EXPECT_NEAR(sqrt(5),
      distancePointLineSegment(Point<int>(2, -1), l1), 1e-10);

  EXPECT_NEAR(2,
      distancePointLineSegment(Point<int>(-2, 0), l1), 1e-10);

  EXPECT_NEAR(2,
      distancePointLineSegment(Point<int>(2, 6), l1), 1e-10);

  EXPECT_NEAR(2,
      distancePointLineSegment(Point<int>(2, 10), l1), 1e-10);

  EXPECT_NEAR(sqrt(13),
      distancePointLineSegment(Point<int>(-2, 13), l1), 1e-10);
}

} // namespace two_d
} // namespace geometry
