#ifndef H_GEOMETRY_TWO_D_POINT
#define H_GEOMETRY_TWO_D_POINT
#include <complex>

namespace geometry {
namespace two_d {

template<typename T>
class Point {
 public:
  Point();
  Point(T x,T y);
  Point(std::complex<T>);

  void operator=(const Point<T>& b) { point = b.point; }

  T dot(const Point<T> &b) const {
    return real(conj(point) * b._point());
  }

  T cross(const Point<T>&b) const {
    return imag(conj(point) * b._point());
  }

  T x() const {
    return point.real();
  }

  T y() const {
    return point.imag();
  }

  void swap() {
    point = std::complex<T>(point.imag(), point.real());
  }

  std::complex<T> inline _point() const { return point; }

  // this will allow casting to long double.
  // We will not support other castings as it may be undesired
  // in conversions
  operator Point<long double>() const {
    return Point<long double>(x(), y());
  }
 private:
  std::complex<T> point;
};


template<typename T>
  Point<T>::Point():point(0, 0) {
  }

template<typename T>
  Point<T>::Point(T x, T y):point(x,y) {
  }

template<typename T>
  Point<T>::Point(std::complex<T> p):point(p) {
  }

template<typename T>
  bool operator==(const Point<T>&a, const Point<T>& b) {
    return a._point() == b._point();
  }

template<typename T>
  bool operator!=(const Point<T>&a, const Point<T>& b) {
    return a._point() != b._point();
  }

template<typename T>
  Point<T> operator+(const Point<T> &a, const Point<T>& b) {
    return Point<T>(a._point() + b._point());
  }

template<typename T>
  Point<T> operator-(const Point<T> &a, const Point<T>& b) {
    return Point<T>(a._point() - b._point());
  }

template<typename T>
  Point<T> operator-(const Point<T> &a) {
    return a * -1;
  }

template<typename T>
  Point<T> operator*(const Point<T> &a, T scalar) {
    return Point<T>(a._point() * scalar);
  }

template<typename T>
  Point<T> operator/(const Point<T> &a, T scalar) {
    return Point<T>(a._point() / scalar);
  }

} // namespace two_d
} // namespace geometry

#endif
