#include "gtest/gtest.h"
#include "point.h"

namespace geometry {
namespace two_d {

TEST(Point, plus)
{
  Point<int> a(2,5);
  Point<int> b(3,7);
  EXPECT_EQ(Point<int>(5,12), a+b);
}

TEST(Point, minus)
{
  Point<int> a(2,5);
  Point<int> b(3,7);
  EXPECT_EQ(Point<int>(-1,-2), a-b);
}

TEST(Point, unary_minus)
{
  EXPECT_EQ(Point<int>(-1,-2), -Point<int>(1,2));
}

TEST(Point, multiply)
{
  Point<int> a(2,5);
  EXPECT_EQ(Point<int>(4,10), a*2);
}

TEST(Point, divide)
{
  Point<int> a(2,5);
  EXPECT_EQ(Point<int>(1,2), a / 2);
}


TEST(Point, dot)
{
  Point<int> a(2, 5);
  Point<int> b(3, 7);
  EXPECT_EQ(2 * 2 + 5 * 5, a.dot(a));
  EXPECT_EQ(2 * 3 + 5 * 7, a.dot(b)); 
}

TEST(Point, cross)
{
  Point<int> a(2, 5);
  Point<int> b(3, 7);
  EXPECT_EQ(0, a.cross(a));
  EXPECT_EQ(0, b.cross(b));
  EXPECT_EQ(2 * 7 - 3 * 5, a.cross(b)); 
  EXPECT_EQ(2 * 7 - 3 * 5, -b.cross(a));
}

} // namespace two_d
} // namespace geometry

