#include "gtest/gtest.h"
#include "template/template.h"
#include "utils/rand/rand.h"
#include "utils/si_units/si_units.h"
#include "interval_trees/simple/simple_max.h"
#include "interval_trees/fenwick/fenwick.h"
#include "utils/benchmark/benchmark.h"

namespace interval_trees {
  
void randomOperationsFenwick(int size, int count)
{
  Rand rand(20);
  fenwick::FenwickMaxTree<int> tree;
  tree.initialize(fenwick::TO_ZERO, size);

  FOR(q, count) {
    int pos = rand.rand(0, size);
    int t = rand.rand(0, q * 100 + 1);
    tree.update(pos, t);

    int r = rand.rand(1, size);
    tree.get_max(r-1);
  }
}

void randomOperationsSimple(int size, int count)
{
  Rand rand(20);
  simple::SimpleMaxTree<int> tree;
  tree.initialize(size);

  FOR(q, count) {
    int pos = rand.rand(0, size);
    int t = rand.rand(0, q * 100 + 1);
    tree.set(pos, t);

    int r = rand.rand(1, size);
    tree.get_max(0, r);
  }
}


TEST(IntervalTrees, Benchmark)
{
  AUTO_BENCHMARK(randomOperationsSimple(10, 1 * Mi));
  AUTO_BENCHMARK(randomOperationsFenwick(10, 1 * Mi));
  printf("\n");
  AUTO_BENCHMARK(randomOperationsSimple(100, 1 * Mi));
  AUTO_BENCHMARK(randomOperationsFenwick(100, 1 * Mi));
  printf("\n");
  AUTO_BENCHMARK(randomOperationsSimple(1 * Ki, 1 * Mi));
  AUTO_BENCHMARK(randomOperationsFenwick(1 * Ki, 1 * Mi));
  printf("\n");
  AUTO_BENCHMARK(randomOperationsSimple(10 * Ki, 1 * Mi));
  AUTO_BENCHMARK(randomOperationsFenwick(10 * Ki, 1 * Mi));
  printf("\n");
  AUTO_BENCHMARK(randomOperationsSimple(100 * Ki, 1 * Mi));
  AUTO_BENCHMARK(randomOperationsFenwick(100 * Ki, 1 * Mi));
  printf("\n");
  AUTO_BENCHMARK(randomOperationsSimple(1 * Mi, 1 * Mi));
  AUTO_BENCHMARK(randomOperationsFenwick(1 * Mi, 1 * Mi));
  printf("\n");
  AUTO_BENCHMARK(randomOperationsSimple(10 * Mi, 1 * Mi));
  AUTO_BENCHMARK(randomOperationsFenwick(10 * Mi, 1 * Mi));
}

} // namespace interval_trees
