#include "fenwick.h"
#include "gtest/gtest.h"
#include "template/template.h"
#include "utils/rand/rand.h"
#include "interval_trees/array/interval_array.h"

namespace interval_trees {
namespace fenwick {

TEST(Fenwick, CompareToArray)
{
  const int TEST_ITERATIONS = 200;
  const int QUERIES = 10000;
  const int MAX = 10000;
  printf("going to test %d iterations with %d queries each "
         "on max %d values\n", TEST_ITERATIONS, QUERIES, MAX);

  Rand rand(20);
  FOR(cnt, TEST_ITERATIONS) {
    FenwickMaxTree<int> fenwick;
    IntervalMaxArray<int> array;

    int n = rand.rand(1, MAX + 1);
    fenwick.initialize(TO_ZERO, n);
    array.initialize(n);

    int k = rand.rand(1, 21);
    FOR(q, QUERIES) {
        if (rand.rand(0, 50) > 0) {
            int pos = rand.rand(0, n);
            int t = rand.rand(0, q*k+1);
            fenwick.update(pos, t);
            array.update(pos, pos+1, t);
        } else {
            int r = rand.rand(1, n);
            EXPECT_EQ(array.get_max(0, r), fenwick.get_max(r-1));
        }
    }
  }
}

} // namespace fenwick
} // namespace interval_trees
