#include "simple_max.h"
#include "interval_trees/array/interval_array.h"
#include "gtest/gtest.h"
#include "template/template.h"
#include "utils/rand/rand.h"

#define TEST_ITERATIONS 200
#define QUERIES 10000

#define MAX 10000

namespace interval_trees {
namespace simple {

TEST(SimpleMax, badInitialization)
{
  EXPECT_THROW(SimpleMaxTree<int> s; s.initialize(0),
               std::invalid_argument);

}

TEST(SimpleMax, badSet)
{
  SimpleMaxTree<int> s;
  s.initialize(10);
  EXPECT_THROW(s.set(10, 0), std::invalid_argument);
  EXPECT_THROW(s.set((unsigned int) (-1), 0), std::invalid_argument);
}

TEST(SimpleMax, badMax)
{
  SimpleMaxTree<int> s;
  s.initialize(10);
  EXPECT_THROW(s.get_max(5, 4), std::invalid_argument);
  EXPECT_THROW(s.get_max(0, 14), std::invalid_argument);
  EXPECT_THROW(s.get_max((unsigned int) -1, 4), std::invalid_argument);
}

TEST(SimpleMax, customDefault)
{
  SimpleMaxTree<int> s;
  s.initialize(10, 47);
  EXPECT_EQ(47, s.get_max(4, 7));
}

TEST(SimpleMax, SmallPermutations)
{
  SimpleMaxTree<int> simple;
  IntervalMaxArray<int> array;
  
  const int N = 7;
  int data[N];
  simple.initialize(N);
  array.initialize(N);
  FOR(i, N) data[i] = i;
  do {
    FOR(i, N) {
      simple.set(i, data[i]);
      array.set(i, i+1, data[i]);
      EXPECT_EQ(data[i], simple.get(i));
    }

    FOR(right, N +1) {
      FOR(left, right) {
        EXPECT_EQ(array.get_max(left, right),
                  simple.get_max(left, right));

      }
    }

  } while (std::next_permutation(data, data + N));
  
}

} // namespace simple
} // namespace interval_trees
