#include <gtest/gtest.h>
#include "factorize_naive.h"
#include "factorize_with_oracle.h"
#include "utils/benchmark/benchmark.h"
#include "utils/si_units/si_units.h"

namespace math {
namespace factorize {

template <class algorithm>
void factorizeRange(long long start, long long  length) {
  for (long long int i = start; i < start + length; i++)
    algorithm::factorize(i);
}

TEST(Factorize, benchmark) {
  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(10, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(10, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(10, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(10 * Ki, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(10 * Ki, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(10 * Ki, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(100 * Ki, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(100 * Ki, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(100 * Ki, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(1 * Mi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(1 * Mi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(1 * Mi, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(10 * Mi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(10 * Mi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(10 * Mi, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(100 * Mi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(100 * Mi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(100 * Mi, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(1 * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(1 * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(1 * Gi, Ki));
  printf("\n");
  
  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(10LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(10LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(10LL * Gi, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(100LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(100LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(100LL * Gi, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(1000LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(1000LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(1000LL * Gi, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(10000LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(10000LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(10000LL * Gi, Ki));
  printf("\n");

  AUTO_BENCHMARK(factorizeRange<FactorizeNaive>(100000LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizePollard>(100000LL * Gi, Ki));
  AUTO_BENCHMARK(factorizeRange<FactorizeBrent>(100000LL * Gi, Ki));
}


} // namespace factorize
} // namespace math
