#ifndef H_MATH_GCD_GCD
#define H_MATH_GCD_GCD

#include <algorithm>
#include <limits>
#include "utils/branch_predict/branch_predict.h"
#include <stdexcept>

namespace math {
namespace gcd {


template<typename T>
  T gcd(T a, T b) {
    // fix the negative values
    if (std::numeric_limits<T>::is_signed && a < 0) {
      if (UNLIKELY(a == std::numeric_limits<T>::min())) {
        throw std::overflow_error("problem calculating gcd - big negative number");
      }
      a = -a;
    }
    if (std::numeric_limits<T>::is_signed && b < 0) {
      if (UNLIKELY(b == std::numeric_limits<T>::min())) {
        throw std::overflow_error("problem calculating gcd - big negative number");
      }
      b = -b;
    }

    // Warning: std::__gcd works only with positive values
    // Note: std::__gcd is for internal use, but we do not want to reimplement
    // already implemented things
    return std::__gcd(a, b);
  }

} // namespace gcd
} // namespace math
#endif
