#include "gtest/gtest.h"
#include "modular_inverse_precomputed.h"
#include "modular_inverse_fermat.h"
#include "modular_inverse_gcd.h"
#include "utils/benchmark/benchmark.h"


namespace math {
namespace modular_inverse {

void benchmarkRange(unsigned int p) {
  ModularInversePrecomputed m;
  m.initialize(p);
  for (unsigned int i = 0; i < p; i++)
    m.getInverse(i);
}


void benchmarkRangeFermat(unsigned int p, unsigned int start, unsigned int stop) {
  for (unsigned int i = start; i < stop; i++)
    ModularInverseFermat_<powermod::PowermodSimple, false>::getInverse(i, p);
}

void benchmarkRangeGcd(unsigned int p, unsigned int start, unsigned int stop) {
  for (unsigned int i = start; i < stop; i++)
    ModularInverseGcd::getInverse((signed int) i, (signed int) p);
}

void benchmarkRangeFermat(unsigned int p) {
  benchmarkRangeFermat(p, 0, p);
}

void benchmarkRangeGcd(unsigned int p) {
  benchmarkRangeGcd(p, 0, p);
}

TEST(ModularInverse, benchmarkSmall) {
  AUTO_BENCHMARK(benchmarkRange(48611));
  AUTO_BENCHMARK(benchmarkRangeFermat(48611));
  AUTO_BENCHMARK(benchmarkRangeGcd(48611));

  AUTO_BENCHMARK(benchmarkRange(195413));
  AUTO_BENCHMARK(benchmarkRangeFermat(195413));
  AUTO_BENCHMARK(benchmarkRangeGcd(195413));
}

TEST(ModularInverse, benchmarkLarge) {
  AUTO_BENCHMARK(benchmarkRange(888061));
  AUTO_BENCHMARK(benchmarkRangeFermat(888061));
  AUTO_BENCHMARK(benchmarkRangeGcd(888061));

  AUTO_BENCHMARK(benchmarkRange(15485863));
  AUTO_BENCHMARK(benchmarkRangeFermat(15485863));
  AUTO_BENCHMARK(benchmarkRangeGcd(15485863));
}

TEST(ModularInverse, benchmarkLognVersions) {
  AUTO_BENCHMARK(benchmarkRangeFermat(982451653, 1000000, 2000000));
  AUTO_BENCHMARK(benchmarkRangeGcd(982451653, 1000000, 2000000));

}


} // namespace modular_inverse
} // namespace math
