#include "gtest/gtest.h"
#include "modular_inverse_precomputed.h"
#include "math/powermod/powermod.h"

namespace math {
namespace modular_inverse {

typedef ModularInversePrecomputed ModInv;

TEST(ModInvTest, invalidInit) {
  EXPECT_THROW(ModInv m; m.initialize(0u), std::invalid_argument);
  EXPECT_THROW(ModInv m; m.initialize(1u), std::invalid_argument);
  EXPECT_THROW(ModInv m; m.initialize(4u), std::invalid_argument);
  EXPECT_THROW(ModInv m; m.initialize(27u), std::invalid_argument);
}

TEST(ModInvTest, small) {
  ModInv m;
  m.initialize(5u);
  EXPECT_EQ(0u, m.getInverse(0u));
  EXPECT_EQ(1u, m.getInverse(1u));
  EXPECT_EQ(3u, m.getInverse(2u));
  EXPECT_EQ(2u, m.getInverse(3u));
  EXPECT_EQ(4u, m.getInverse(4u));
  EXPECT_THROW(m.getInverse(5u), std::invalid_argument);
}

TEST(ModInvTest, consistency) {
  unsigned int primes[] = {2, 3, 5, 7, 11, 13,
    4987, 43991, 77477, 195413, 293173,
    435371, 568999, 888061,
    0};

  for (int i = 0; primes[i] != 0; i++) {
    unsigned int p = primes[i];
    ModInv m;
    m.initialize(p);
    EXPECT_EQ(0u, m.getInverse(0u));
    for (unsigned int j = 1; j < p; j++ ) {
      unsigned int inverse = m.getInverse(j);
      EXPECT_EQ(1u, math::powermod::PowermodSimple::multmod(j, inverse, p));
    }
  }

}

} // namespace modular_inverse
} // namespace math
