#ifndef H_MATH_POWERMOD_MULTMOD_SIMPLE
#define H_MATH_POWERMOD_MULTMOD_SIMPLE

#include "utils/static_assert/static_assert.h"
#include "utils/preconditions/preconditions.h"
#include <limits>
#include "utils/macros/unused.h"

namespace math {
namespace powermod {

class MultmodSimple {
  typedef int BaseType;
  typedef long long DoubleType;
  STATIC_ASSERT(std::numeric_limits<BaseType>::digits * 2 <=
                std::numeric_limits<DoubleType>::digits,
                "The long long is not enough for temporary "
                "computations");

    public:
  template <typename T>
  static T max_argument(T UNUSED(x)) {
    if (std::numeric_limits<BaseType>::digits >
        std::numeric_limits<T>::digits) {
      return std::numeric_limits<T>::max();
    } else {
      return std::numeric_limits<BaseType>::max();
    }
  }

  static BaseType multmod(BaseType a, BaseType b, BaseType modulo) {
    Preconditions::check(modulo > 0, "Modulo should be positive.");
    Preconditions::checkRange(a, 0, modulo);
    Preconditions::checkRange(b, 0, modulo);

    return ((DoubleType) a * (DoubleType) b) % (DoubleType) modulo;
  }
};

} // namespace powermod
} // namespace math

#endif
