#ifndef H_MATH_POWERMOD
#define H_MATH_POWERMOD
/**
 * This file contains fast computation of power of a to b modulo m.
 */

#include "utils/static_assert/static_assert.h"
#include "utils/preconditions/preconditions.h"
#include <limits>
#include "multmod_simple.h"
#include "multmod_extended.h"

namespace math {
namespace powermod {

# pragma GCC diagnostic ignored "-Wtype-limits"
template <class MultModImpl>
class Powermod_ {
  public:
  template <typename T>
    static T powermod(T base, T power, T modulo) {
    Preconditions::check(modulo > 0, "Modulo should be positive.");
    Preconditions::check(power >= 0, "We don't support modular inverses.");
    Preconditions::checkRange(base, modulo);
    const T max = MultModImpl::max_argument((T) 0); 
    Preconditions::check(modulo <= max, "Too big argument for powermod");
    // Beware of special case power==0 && modulo==1 (result is 0 not 1)
    if (modulo == 1) {
      return 0;
    }
    T result = 1;
    T current = base;
    while (power > 0) {
      if (power % 2) {
        result = MultModImpl::multmod(result, current, modulo);
      }
      current = MultModImpl::multmod(current, current, modulo);
      power /= 2;
    }
    return result;
  }

  template <typename T>
  static T multmod(T a, T b, T modulo) {
    Preconditions::check(modulo > 0, "Modulo should be positive.");
    Preconditions::checkRange(a, modulo);
    Preconditions::checkRange(b, modulo);
    const T max = MultModImpl::max_argument((T) 0);
    Preconditions::check(modulo <= max, "Too big argument for multmod");

    return MultModImpl::multmod(a, b, modulo);
  }
};

typedef Powermod_<MultmodSimple> PowermodSimple;
typedef Powermod_<MultmodExtendedOpt> PowermodExtended;

} // namespace powermod
} // namespace math

#endif
