#include "gtest/gtest.h"
#include "segmented_sieve.h"
#include "math/primes/primes_test_data.h"
#include "utils/macros/unused.h"

namespace math {
namespace prime_sieve {

class CountPrimesCallback: public SieveCallback {
 public:
  virtual void foundPrime(long long int UNUSED(x)) {
    cnt++;
  }

  int getCount() {
    return cnt;
  }

  void reset() {
    cnt = 0;
  }
 private:
   int cnt;
};

class CountTwinsCallback: public SieveCallback {
 public:
  virtual void foundPrime(long long int x) {
    if (x - last == 2) {
      cnt++;
    }
    last = x;
  }

  int getCount() {
    return cnt;
  }

  void reset() {
    cnt = 0;
    // Warning: don't use zero here, as (0,2) will be identified
    // as twin.
    last = -1;
  }
 private:
   int cnt;
   int last;
};

TEST(SegmentedSieve, counts) {
  CountPrimesCallback c;

  for (int i = 0; testdata::prime_count_small[i][0] > 0; i++) {
    c.reset();
    SegmentedSieve::findPrimes(testdata::prime_count_small[i][0], &c);
    EXPECT_EQ(testdata::prime_count_small[i][1], c.getCount());
  }
}

TEST(SegmentedSieve, countsBig) {
  CountPrimesCallback c;
  long long int upper;
  for (int i = 0; (upper = testdata::prime_count_big[i][0]) > 0; i++) {
    printf("testing %lld\n", upper);
    c.reset();
    SegmentedSieve::findPrimes(upper, &c);
    EXPECT_EQ(testdata::prime_count_big[i][1], c.getCount());
  }
}

TEST(SegmentedSieve, twins) {
  CountTwinsCallback c;

  for (int i = 0; testdata::prime_twins_count[i][0] > 0; i++) {
    printf("testing %lld\n", testdata::prime_twins_count[i][0]);
    c.reset();
    SegmentedSieve::findPrimes(testdata::prime_twins_count[i][0], &c);
    EXPECT_EQ(testdata::prime_twins_count[i][1], c.getCount());
  }
}

} // namespace prime_sieve
} // namespace math
