#include "gtest/gtest.h"
#include "math/prime_sieve/eratosthenes_basic.h"
#include "math/prime_sieve/eratosthenes_optimized.h"
#include "primes_basic.h"
#include "primes_slow.h"
#include "primes_fast.h"

using namespace math::prime_sieve;

namespace math {
namespace primes {

TEST(Compare, compareEratosthenes) {
  EratosthenesBasic eb;
  EratosthenesOptimized eo;

  int sizes[] = {10, 100, 1000, 10000, 100000, 1000000, 10000000, 0};
  for (int i = 0; sizes[i] > 0; i++) {
    unsigned int size = sizes[i];
    printf("comparing eratosthenes implementations for size %d\n",
        size);
    eb.initialize(size);
    eo.initialize(size);
    for (unsigned int j = 0; j < size; j++) {
      EXPECT_EQ(eb.isPrime(j), eo.isPrime(j));
    }
  }
}

TEST(Compare, comparePrimes) {
  const unsigned int MAX = 20000;
  printf("comparing isPrime implementations up to %d\n", MAX);
  for (unsigned int i = 1; i < MAX; i++) {
    EXPECT_EQ(PrimesSlow::isPrime(i), PrimesBasic::isPrime(i));
  }
}


TEST(Compare, comparePrimesBasicEratosthenesOptimized) {
  const unsigned int MAX = 1000000;
  printf("comparing basic isPrime implementation to "
      " optimized eratosthenes up to %d\n", MAX);
  EratosthenesOptimized eo;
  eo.initialize(MAX);

  for (unsigned int i = 1; i < MAX; i++) {
    EXPECT_EQ(eo.isPrime(i), PrimesBasic::isPrime(i));
  }
}

TEST(Compare, comparePrimesFastEratosthenesOptimized) {
  const unsigned int MAX = 100000000; // 100M
  printf("comparing fast isPrime implementation to "
      " optimized eratosthenes up to %d\n", MAX);
  EratosthenesOptimized eo;
  eo.initialize(MAX);
  printf("eratosthenes initialized\n");

  for (unsigned int i = 1; i < MAX; i++) {
    EXPECT_EQ(eo.isPrime(i), PrimesFast::isPrime(i));
  }
}

} // namespace primes
} // namespace math
