#include "gtest/gtest.h"
#include "primes_fast.h"
#include <set>

namespace math {
namespace primes {


TEST(PrimesFast, invalidRange) {
  EXPECT_THROW(PrimesFast::isPrime(0),
               std::invalid_argument);
  EXPECT_THROW(PrimesFast::isPrime(-1),
               std::invalid_argument);
  EXPECT_THROW(PrimesFast::isPrime(-47),
               std::invalid_argument);
}

TEST(PrimesFast, smallNumbers) {
  EXPECT_EQ(0, PrimesFast::isPrime(1));
  EXPECT_EQ(true, PrimesFast::isPrime(2));
  EXPECT_EQ(true, PrimesFast::isPrime(3));
  EXPECT_EQ(0, PrimesFast::isPrime(4));
  EXPECT_EQ(true, PrimesFast::isPrime(5));
}

TEST(PrimesFast, bigNumbers) {
  int low = 104729;
  int high = 104774;
  std::set<int> primes;
  primes.insert(104729);
  primes.insert(104743);
  primes.insert(104759);
  primes.insert(104761);
  primes.insert(104773);

  for (int i = low; i<high; i++) {
    EXPECT_EQ(primes.find(i) != primes.end(),
        PrimesFast::isPrime(i));
  }
}

TEST(PrimesFast, squares) {
  const int MAX = 46340; // sqrt(2^31);
  for (int i = 2; i < MAX; i++) {
    EXPECT_FALSE(PrimesFast::isPrime(i * i));
  }
}

} // namespace primes
} // namespace math
