#ifndef H_MATH_PRIMES_PRIMES_SLOW
#define H_MATH_PRIMES_PRIMES_SLOW
/**
 * @file Contains extremely slow implementation (O(n)) of primality testing.
 *
 * @warning: This is terribly slow! Use other methods instead, this is implementation is just
 * for testing.
 */
#include "utils/preconditions/preconditions.h"
#include "utils/static_assert/static_assert.h"
#include <limits>

namespace math {
namespace primes {

/**
 * Very naive implementation of primality testing.
 *
 * @warning: This is terribly slow! Use other methods instead, this is implementation is just
 * for testing.
 */
class PrimesSlow {
public:
  /**
   * Checks whether a specified number is prime
   *
   * @precondition \a BaseType is integral typpe
   *
   * @param p number to be tested
   *
   * @returns true iff p is a prime number
   */
  template <typename BaseType>
  bool static isPrime(BaseType p) {
    STATIC_ASSERT(std::numeric_limits<BaseType>::is_integer, 
        "BaseType should be integer");
    Preconditions::check(p > 0, "tested integer should be positive");
    if (p == 1) {
      return false;
    }
    for (BaseType i = 2; i < p; i++) {
      if (p % i == 0) {
        return false;
      }
    }
    return true;
  }
};

} // namespace primes
} // namespace math
#endif
