#include "gtest/gtest.h"
#include "primes_slow.h"
#include "primes_basic.h"
#include <set>

namespace math {
namespace primes {

TEST(PrimesSlow, invalidRange) {
  EXPECT_THROW(PrimesSlow::isPrime(0), std::invalid_argument);
  EXPECT_THROW(PrimesSlow::isPrime(-1), std::invalid_argument);
  EXPECT_THROW(PrimesSlow::isPrime(-47), std::invalid_argument);
}

TEST(PrimesSlow, smallNumbers) {
  EXPECT_FALSE(PrimesSlow::isPrime(1));
  EXPECT_EQ(true, PrimesSlow::isPrime(2));
  EXPECT_EQ(true, PrimesSlow::isPrime(3));
  EXPECT_FALSE(PrimesSlow::isPrime(4));
  EXPECT_EQ(true, PrimesSlow::isPrime(5));
}

TEST(PrimesSlow, bigNumbers) {
  int low = 104729;
  int high = 104774;
  std::set<int> primes;
  primes.insert(104729);
  primes.insert(104743);
  primes.insert(104759);
  primes.insert(104761);
  primes.insert(104773);

  for (int i = low; i<high; i++) {
    EXPECT_EQ(primes.find(i) != primes.end(),
        PrimesSlow::isPrime(i));
  }
}

TEST(PrimesBasic, invalidRange) {
  EXPECT_THROW(PrimesBasic::isPrime(0), std::invalid_argument);
  EXPECT_THROW(PrimesBasic::isPrime(-1), std::invalid_argument);
  EXPECT_THROW(PrimesBasic::isPrime(-47), std::invalid_argument);
}

TEST(PrimesBasic, smallNumbers) {
  EXPECT_FALSE(PrimesSlow::isPrime(1));
  EXPECT_EQ(true, PrimesSlow::isPrime(2));
  EXPECT_EQ(true, PrimesSlow::isPrime(3));
  EXPECT_FALSE(PrimesSlow::isPrime(4));
  EXPECT_EQ(true, PrimesSlow::isPrime(5));
}

TEST(PrimesBasic, bigNumbers) {
  int low = 982451652;
  int high = 982451820;
  std::set<int> primes;
  primes.insert(982451653);
  primes.insert(982451707);
  primes.insert(982451737);
  primes.insert(982451809);
  primes.insert(982451819);

  for (int i = low; i<high; i++) {
    EXPECT_EQ(primes.find(i) != primes.end(),
        PrimesBasic::isPrime(i));
  }
}



TEST(PrimesBasic, squares) {
  for (int i = 2; i < 100000; i++) {
    EXPECT_FALSE(PrimesBasic::isPrime(i* (long long int) i));
  }
}

TEST(PrimesSlow, count1) {
  const int low = 2;
  const int high = 10000;
  const int expected_cnt = 1229;
  int cnt = 0;
  for (int i = low; i < high; i++)  {
    cnt += PrimesSlow::isPrime(i);
  }
  EXPECT_EQ(expected_cnt, cnt);

}

} // namespace primes
} // namespace math
