#include "gtest/gtest.h"
#include "duval.h"
#include "debug/ppdebug.h"
#include <cstring>
#include "utils/static_assert/static_assert.h"
#include "utils/macros/array_size.h"
#include "utils/rand/rand.h"

namespace strings {
namespace cyclic {

TEST(Duval, simpleSuffixes) {
  std::vector<int> out;
  char c1[] = "abrakadabra";
  int expected1[] = {0, 0, 0, 3, 0, 5, 0, 7, 7, 7, 10};
  ASSERT_EQ(ARRAY_SIZE(expected1), strlen(c1));

  Duval<char>::minimumSuffixes(c1, strlen(c1), &out);
  EXPECT_EQ(ARRAY_SIZE(expected1), out.size());
  for (unsigned int i = 0; i < ARRAY_SIZE(expected1); i++) {
    EXPECT_EQ(expected1[i], out[i]);
  }

  // next test

  char c2[] = "3141592653";
  int expected2[] = {0, 1, 1, 3, 1, 1, 1, 1, 1, 1};
  ASSERT_EQ(ARRAY_SIZE(expected2), strlen(c2));

  Duval<char>::minimumSuffixes(c2, strlen(c2), &out);
  EXPECT_EQ(ARRAY_SIZE(expected2), out.size());
  for (unsigned int i = 0; i < ARRAY_SIZE(expected2); i++) {
    EXPECT_EQ(expected2[i], out[i]);
  }
  // next test

  char c3[] = "aaaaaaaa";
  int expected3[] = {0, 1, 2, 3, 4, 5, 6, 7};
  ASSERT_EQ(ARRAY_SIZE(expected3), strlen(c3));

  Duval<char>::minimumSuffixes(c3, strlen(c3), &out);
  EXPECT_EQ(ARRAY_SIZE(expected3), out.size());
  for (unsigned int i = 0; i < ARRAY_SIZE(expected3); i++) {
    EXPECT_EQ(expected3[i], out[i]);
  }
}

TEST(Duval, suffixConsistency) {
  Rand r(42);
  const unsigned int SIZE = 5000;
  const unsigned int MAX = 100;
  vector<int> reference;
  vector<int> out;
  vector<int> input;
  for (unsigned int i = 0; i < SIZE; i++) {
    input.push_back(r.rand(0, MAX));
  }


  Duval<int>::minimumSuffixes(&input[0], SIZE, &reference);
  ASSERT_EQ(SIZE, reference.size());

  while (!input.empty()) {
    Duval<int>::minimumSuffixes(&input[0], input.size(), &out);
    ASSERT_EQ(input.size(), out.size());
    for (int j = 0; j < (int)input.size(); j++) {
      EXPECT_EQ(reference[j], out[j]);
    }
    input.pop_back();
  }
}

TEST(Duval, simpleCyclicShift) {
  const char* testdata[] = {
    "abrakadabra",
    "aaaaaaaaaa",
    "abcabc",
    "cabcab",
    0
  };

  int expected[] = {
    10,
    0,
    0,
    1,
    -1
  };
  STATIC_ASSERT(ARRAY_SIZE(testdata) == ARRAY_SIZE(expected), "");
  for (int i = 0; testdata[i] != 0; i++) {
    ASSERT_EQ(expected[i],
        Duval<const char>::leastCyclicShift(testdata[i], strlen(testdata[i])));
  }
}

} // namespace cyclic
} // namespace strings
