#include <cstring>
#include "gtest/gtest.h"
#include "lcs.h"
#include "lcs_hirschberg.h"
#include "utils/macros/array_size.h"
#include "utils/static_assert/static_assert.h"

namespace strings {
namespace lcs {

TEST(LCS, simple) {
  const char* testdata[][2] = {
    {"", ""},
    {"", "ABCD"},
    {"ABCD", ""},
    {"A", "ABCD"},
    {"ABCD", "B"},
    {"ABCD", "ABCD"},
    {"ABCD", "DCBA"},
    {"XMJYAUZ", "MZJAWXU"},
    {NULL, NULL}};
  int expected[] = {
    0, // , 
    0, // ,ABCD
    0, // ABCD,
    1, // A,ABCD
    1, // ABCD,B
    4, // ABCD,ABCD
    1, // ABCD,DCBA
    4,
    -1};

  STATIC_ASSERT(ARRAY_SIZE(testdata) == ARRAY_SIZE(expected), "");
  for (int i = 0; testdata[i][0]; i++) {
    printf("testing %s %s\n", testdata[i][0], testdata[i][1]);
    EXPECT_EQ(expected[i],
        LCS<char>::length(
          testdata[i][0], strlen(testdata[i][0]),
          testdata[i][1], strlen(testdata[i][1])));
        
  }
}

TEST(LCSHirschberg, length) {
  const char* testdata[][2] = {
    {"", ""},
    {"", "ABCD"},
    {"ABCD", ""},
    {"A", "ABCD"},
    {"ABCD", "B"},
    {"ABCD", "ABCD"},
    {"ABCD", "DCBA"},
    {"XMJYAUZ", "MZJAWXU"},
    {NULL, NULL}};
  int expected[] = {
    0, // , 
    0, // ,ABCD
    0, // ABCD,
    1, // A,ABCD
    1, // ABCD,B
    4, // ABCD,ABCD
    1, // ABCD,DCBA
    4,
    -1};

  STATIC_ASSERT(ARRAY_SIZE(testdata) == ARRAY_SIZE(expected), "");
  for (int i = 0; testdata[i][0]; i++) {
    printf("testing %s %s\n", testdata[i][0], testdata[i][1]);
    vector<char> out;
    LCSHirschberg<char>::subsequence(
      testdata[i][0], strlen(testdata[i][0]),
      testdata[i][1], strlen(testdata[i][1]), &out);
    EXPECT_EQ(expected[i], (int) out.size());
        
  }
}

} // lcs
} // strings
