#include "gtest/gtest.h"
#include "strings/suffix_array_naive/naive.h"
#include "strings/utils/sequence_loader.h"
#include "utils/benchmark/benchmark.h"
#include "debug/ppdebug.h"
#include "strings/testdata.h"
#include <string>
#include "strings/search_callback/search_callback.h"
#include "strings/search_rabin_karp/rabin_karp.h"
#include "strings/search_kmp/kmp.h"
#include "strings/suffix_array_binsearch/binsearch.h"
#include "utils/macros/unused.h"
#include "strings/suffix_array_myers/manber_myers.h"
namespace strings {
namespace search {

class CountCallback: public strings::search_callback::SearchCallback<
                      std::vector<unsigned char>::iterator >
{
  public:
  int cnt;

  CountCallback() {
    cnt = 0;
  }

  virtual void foundMatch(const std::vector<unsigned char>::iterator& UNUSED(it)) {
    cnt++;
  }
};

/**
 * Return files on which we want to compare
 * results of implementations.
 */
std::vector<std::string> getTestFiles() {
  std::string testdir = "../";
  std::vector<std::string> files;
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_RANDOM);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_PI);
  files.push_back(testdir + TestdataFiles::TEXT_BIBLE);
  files.push_back(testdir + TestdataFiles::TEXT_FACTBOOK);
  files.push_back(testdir + TestdataFiles::GENOME_ECOLI);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_ALPHABET_SMALL);
  files.push_back(testdir + TestdataFiles::TEXT_APACHE_LOGS);
  files.push_back(testdir + TestdataFiles::GENOME_CHROMOSOME_Y);
  files.push_back(testdir + TestdataFiles::SOURCE_CODE_PHP);
  return files;
}

TEST(SearchBenchmark, benchmark) {

  std::map<unsigned char, unsigned char> mapa;
  for (int c = 0; c < 256; c++) {
    mapa[(unsigned char) c] = (unsigned char) c;
  }

  std::vector<std::string> files = getTestFiles();
  for (size_t file_no = 0; file_no < files.size(); file_no++) {
    std::vector<unsigned char> sequence;
    std::string filename = files[file_no];
    printf(">>>>>>>>>>>>>>>>>loading sequence %s\n", filename.c_str());
    sequence = utils::SequenceLoader::loadSequence(filename.c_str(), mapa);
    std::vector<int> suf_result;
    AUTO_BENCHMARK(suffix_array::ManberMyers::buildSuffixArray(
        sequence.begin(), sequence.end(), 300, &suf_result));
    printf("\n");

    int pattern_no = 0;
    while (PatternFiles::SEARCH_PATTERNS[pattern_no] != NULL) {
      std::string base = "../";
      std::string patternFilename = 
        base + PatternFiles::SEARCH_PATTERNS[pattern_no];
      printf("pattern %s\n", patternFilename.c_str());
      std::vector<unsigned char> pattern =
        utils::SequenceLoader::loadSequence(patternFilename.c_str(), mapa);

      CountCallback callback;

      AUTO_BENCHMARK(RabinKarp::search(sequence.begin(), sequence.end(),
          pattern.begin(), pattern.end(), true, &callback));
      AUTO_BENCHMARK(RabinKarp::search(sequence.begin(), sequence.end(),
          pattern.begin(), pattern.end(), false, &callback));

      AUTO_BENCHMARK(KMP::search(sequence.begin(), sequence.end(),
          pattern.begin(), pattern.end(), &callback));
      callback.cnt = 0;

      AUTO_BENCHMARK(suffix_array::Binsearch::searchSuffixArray(
                  sequence.begin(), sequence.end(),
                  suf_result, pattern.begin(), pattern.end(),
                  &callback));
      printf("matches %d\n\n", callback.cnt);
      pattern_no++;
    }
  }
}

} // namespace suffix_array
} // namespace strings
