#include "gtest/gtest.h"
#include "strings/suffix_array_naive/naive.h"
#include "strings/utils/sequence_loader.h"
#include "utils/benchmark/benchmark.h"
#include "debug/ppdebug.h"
#include "strings/testdata.h"
#include <string>
#include "strings/search_callback/search_callback.h"
#include "kmp.h"
#include "utils/macros/unused.h"

namespace strings {
namespace search {

class CountCallback: public strings::search_callback::SearchCallback<
                      std::vector<unsigned char>::iterator >
{
  public:
  int cnt;
  const char* expected;
  int expected_len;

  CountCallback(const char* expected_) {
    cnt = 0;
    expected = expected_;
    expected_len = strlen(expected);
  }
    
  virtual void foundMatch(const std::vector<unsigned char>::iterator& it) {
    const char* data = (const char*)(&(*it));
    cnt++;
    EXPECT_EQ(0, strncmp(expected, data, expected_len));
  }
};

TEST(KMP, test) {

  std::map<unsigned char, unsigned char> mapa;
  for (int c = 0; c < 256; c++) {
    mapa[(unsigned char) c] = (unsigned char) c;
  }
  std::string base = "../";
  std::vector<unsigned char> sequence;
  std::vector<int> suf_result;
  printf("loading sequence\n");
  sequence = utils::SequenceLoader::loadSequence((base + TestdataFiles::TEXT_BIBLE).c_str(), mapa);

  const char *tmp = "and";
  const int pat_len = strlen(tmp);
  const unsigned char* pat = (const unsigned char*) tmp;
  CountCallback callback(tmp);;

  printf("searching\n");
  KMP::search(sequence.begin(), sequence.end(),
        pat, pat + pat_len, &callback);

  printf("found %d matches\n", callback.cnt);
  const int COUNT_AND_IN_BIBLE = 43878;
  EXPECT_EQ(COUNT_AND_IN_BIBLE, callback.cnt);
}

} // namespace suffix_array
} // namespace strings
