#include "gtest/gtest.h"
#include <cstdio>
#include "strings/utils/sequence_loader.h"
#include "strings/testdata.h"
#include "lcp_kasai.h"
#include "strings/suffix_array_log2/manber_myers_log2.h"
#include "utils/benchmark/benchmark.h"
#include <numeric>
#include <algorithm>

namespace strings {
namespace suffix_array {
/**
 * Return files on which we want to compare
 * results of implementations.
 */
std::vector<std::string> getTestFiles() {
  std::string testdir = "../";
  std::vector<std::string> files;
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_RANDOM);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_PI);
  files.push_back(testdir + TestdataFiles::TEXT_BIBLE);
  files.push_back(testdir + TestdataFiles::TEXT_FACTBOOK);
  files.push_back(testdir + TestdataFiles::GENOME_ECOLI);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_ALPHABET_SMALL);
  files.push_back(testdir + TestdataFiles::TEXT_APACHE_LOGS);
  files.push_back(testdir + TestdataFiles::GENOME_CHROMOSOME_Y);
  files.push_back(testdir + TestdataFiles::SOURCE_CODE_PHP);
  return files;
}


TEST(LCPNaive, benchmark) {
  // load all characters
  std::map<unsigned char, unsigned char> mapa;
  for (int c = 0; c < 256; c++) {
    mapa[(unsigned char) c] = (unsigned char) c;
  }

  std::vector<std::string> files = getTestFiles();

  for (unsigned int i = 0; i < files.size(); i++) {
    std::vector<unsigned char> sequence;


    printf("loading sequence %s\n", files[i].c_str());
    sequence = utils::SequenceLoader::loadSequence(files[i].c_str(), mapa);
    printf("sequence %s loaded, length %lu\n",
        files[i].c_str(), (long int) sequence.size());

    std::vector<int> sa;

    printf("building suffix array\n");
    ManberMyersLog2::buildSuffixArray(sequence.begin(), sequence.end() , &sa);

    std::vector<int> lcp_naive;

    printf("benchmarking\n");
    BENCHMARK(1,
    LCPKasai::getHeightArray(sequence.begin(), sequence.end(),
        sa.begin(), sa.end(), &lcp_naive));

    long long int sum = std::accumulate(lcp_naive.begin() + 1, lcp_naive.end(), 0LL);
    int max = *std::max_element(lcp_naive.begin() + 1, lcp_naive.end());
    printf("Average LCP: %.2lf\n", (double) sum / (lcp_naive.size() - 1));
    printf("max LCP: %d\n", max);
  }
}

} // namespace suffix_array
} // namespace strings
