#include "gtest/gtest.h"
#include "lcp_kasai.h"
#include "debug/ppdebug.h"
#include <cstdio>
#include "strings/utils/sequence_loader.h"
#include "strings/testdata.h"
#include "strings/suffix_array_lcp_naive/lcp_naive.h"
#include "strings/suffix_array_naive/naive.h"

namespace strings {
namespace suffix_array {
/**
 * Return files on which we want to compare
 * results of implementations.
 */
std::vector<std::string> getTestFiles() {
  std::string testdir = "../";
  std::vector<std::string> files;
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_RANDOM);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_PI);
  files.push_back(testdir + TestdataFiles::TEXT_BIBLE);
  files.push_back(testdir + TestdataFiles::TEXT_FACTBOOK);
  files.push_back(testdir + TestdataFiles::GENOME_ECOLI);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_ALPHABET_SMALL);
  files.push_back(testdir + TestdataFiles::TEXT_APACHE_LOGS);
  return files;
}


TEST(LCPKasai, compareWithNaive) {
  // load all characters
  std::map<unsigned char, unsigned char> mapa;
  for (int c = 0; c < 256; c++) {
    mapa[(unsigned char) c] = (unsigned char) c;
  }

  vector<string> files = getTestFiles();

  for (unsigned int i = 0; i < files.size(); i++) {
    std::vector<unsigned char> sequence;


    printf("loading sequence %s\n", files[i].c_str());
    sequence = utils::SequenceLoader::loadSequence(files[i].c_str(), mapa);
    printf("sequence %s loaded, length %lu\n",
        files[i].c_str(), (unsigned long) sequence.size());

    for (int i = 1; i < 10; i++) {
      printf("comparing lcp arrays on first %lu characters\n",
          (unsigned long) sequence.size() / i);

      std::vector<unsigned char>::iterator end =
          sequence.begin() + (sequence.size() / i);
      std::vector<int> sa;

      NaiveSuffixArray::buildSuffixArray(sequence.begin(), end , &sa);

      std::vector<int> lcp_naive;
      std::vector<int> lcp_kasai;

      LCPNaive::getHeightArray(sequence.begin(), end,
          sa.begin(), sa.end(), &lcp_naive);

      LCPKasai::getHeightArray(sequence.begin(), end,
          sa.begin(), sa.end(), &lcp_kasai);

      lcp_naive[0] = lcp_kasai[0] = 47; // undefined value

      EXPECT_EQ(lcp_naive, lcp_kasai);
    }
  }
}

} // namespace suffix_array
} // namespace strings
