#include "gtest/gtest.h"
#include "strings/utils/sequence_loader.h"
#include "utils/benchmark/benchmark.h"
#include "debug/ppdebug.h"
#include "strings/suffix_array_naive/naive.h"
#include "strings/suffix_array_myers/manber_myers.h"
#include <vector>
#include <string>
#include "strings/testdata.h"

namespace strings {
namespace suffix_array {

/**
 * Return files on which we want to compare
 * results of implementations.
 */
std::vector<std::string> getTestFiles() {
  std::string testdir = "../";
  std::vector<std::string> files;
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_RANDOM);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_PI);
  files.push_back(testdir + TestdataFiles::TEXT_BIBLE);
  files.push_back(testdir + TestdataFiles::TEXT_FACTBOOK);
  files.push_back(testdir + TestdataFiles::GENOME_ECOLI);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_ALPHABET_SMALL);

  return files;
}


TEST(NaiveSuffixArray, compareToNaive) {
  // load all characters
  std::map<unsigned char, unsigned char> mapa;
  for (int c = 0; c < 256; c++) {
    mapa[(unsigned char) c] = (unsigned char) c;
  }

  vector<string> files = getTestFiles();
  for (unsigned int i = 0; i < files.size(); i++) {
    std::vector<unsigned char> sequence;

    std::vector<int> naive_result;
    std::vector<int> myers_result;

    printf("loading sequence %s\n", files[i].c_str());
    sequence = utils::SequenceLoader::loadSequence(files[i].c_str(), mapa);
    printf("sequence %s loaded, length %lu\n",
        files[i].c_str(), (unsigned long) sequence.size());

    for (int i = 1; i < 10; i++) {
      printf("comparing suffix arrays on first %lu characters\n",
         (unsigned long) sequence.size() / i);

      std::vector<unsigned char>::iterator end =
          sequence.begin() + (sequence.size() / i);

      NaiveSuffixArray::buildSuffixArray(sequence.begin(), end , &naive_result);
      ManberMyers::buildSuffixArray(sequence.begin(), end, 256, &myers_result);

      EXPECT_EQ(naive_result, myers_result);
    }
  }
}

} // namespace suffix_array
} // namespace strings

