#include "gtest/gtest.h"
#include "strings/utils/sequence_loader.h"
#include "utils/benchmark/benchmark.h"
#include "debug/ppdebug.h"
#include "strings/suffix_array_log2/manber_myers_log2.h"
#include <vector>
#include <string>
#include "strings/testdata.h"
#include "utils/macros/foreach.h"

namespace strings {
namespace suffix_array {

/**
 * Return files on which we want to compare
 * results of implementations.
 */
std::vector<std::string> getTestFiles() {
  std::string testdir = "../";
  std::vector<std::string> files;
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_RANDOM);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_PI);
  files.push_back(testdir + TestdataFiles::TEXT_BIBLE);
  files.push_back(testdir + TestdataFiles::TEXT_FACTBOOK);
  files.push_back(testdir + TestdataFiles::GENOME_ECOLI);
  files.push_back(testdir + TestdataFiles::ARTIFICIAL_ALPHABET_SMALL);
  files.push_back(testdir + TestdataFiles::TEXT_APACHE_LOGS);
  files.push_back(testdir + TestdataFiles::GENOME_CHROMOSOME_Y);
  files.push_back(testdir + TestdataFiles::SOURCE_CODE_PHP);

  return files;
}


TEST(ManberMyersLog2, benchmark) {
  // load all characters
  std::map<unsigned char, unsigned char> mapa;
  for (int c = 0; c < 256; c++) {
    mapa[(unsigned char) c] = (unsigned char) c;
  }

  vector<string> files = getTestFiles();

  FOREACH(it, files) {
    std::vector<unsigned char> sequence;

    std::vector<int> myers_result;

    printf("loading sequence %s\n", it->c_str());
    sequence = utils::SequenceLoader::loadSequence(it->c_str(), mapa);
    printf("sequence %s loaded, length %lu\n", it->c_str(),
        (unsigned long) sequence.size());

    BENCHMARK(1, 
      ManberMyersLog2::buildSuffixArray(
        sequence.begin(), sequence.end(), &myers_result)
    );
  }
}

} // namespace suffix_array
} // namespace strings

