#include "gtest/gtest.h"
#include "manber_myers_log2.h"
#include "debug/ppdebug.h"
#include <cstdio>

namespace strings {
namespace suffix_array {


TEST(ManberMyersLog2, buildSuffixesSimple) {
  const unsigned int N = 11;
  char str[N + 1] = "abracadabra";
  vector<int> result;

  ManberMyersLog2::buildSuffixArray(str, str + N, &result);

  int expected[N] = {10, 7, 0, 3, 5, 8, 1, 4, 6, 9, 2};
  ASSERT_EQ(N, result.size());
  for (unsigned int i = 0; i < N; i++) {
    ASSERT_EQ(expected[i], result[i]);
  }
}

TEST(ManberMyersLog2, repeatedA) {
  const unsigned int N = 10000;
  char data[N];
  for (unsigned int i = 0; i < N; i++) {
    data[i] = 'A';
  }

  vector<int> result;

  ManberMyersLog2::buildSuffixArray(data, data + N, &result);

  ASSERT_EQ(N, result.size());
  for (unsigned int i = 0; i < N; i++) {
    ASSERT_EQ((int) (N - i - 1), result[i]);
  }
}

TEST(ManberMyersLog2, repeatedAlphabet) {
  std::string data;
  std::string pattern = "abcde";
  for (int i = 0; i < 10000; i++) {
    data += pattern;
  }

  vector<int> result;

  ManberMyersLog2::buildSuffixArray(
      data.begin(), data.end(), &result);

  ASSERT_EQ(data.size(), result.size());
  // expected result for "abcdeabcdeabcde" is
  // [ 10, 5, 0, 11, 6, 1, 12, 7, 2, 13, 8, 3, 14, 9, 4, ]
  for (unsigned int i = 0; i < data.size(); i++) {
    int L = pattern.size();
    int T = data.size() / L; // how many times we have pattern
    int expected = i / T + (T - 1 - (i % T)) * L;
    ASSERT_EQ(expected, result[i]);
  }
}

} // namespace suffix_array
} // namespace strings
