#ifndef H_STRINGS_SUFFIX_ARRAY_NAIVE_NAIVE
#define H_STRINGS_SUFFIX_ARRAY_NAIVE_NAIVE

#include <stdio.h>
#include "utils/preconditions/preconditions.h"
#include <vector>
#include <iterator>
#include <algorithm>
#include "sort_helper.h"

/**
 * Implementation of the suffix array creation by naive 
 * sorting method.
 *
 * Time: O(n^2 log n) worst case when there are suffixes
 * with long same prefixes
 *
 * Warning: This is only internal testing implementation
 * and running time might be really big. Use other implementations
 * instead.
 */
namespace strings {
namespace suffix_array {

class NaiveSuffixArray {
 public:
  /**
   * Build suffix array with naive sorting.
   *
   * _Iterator random access iterator
   *
   * @param input sequence
   * @param length length of the sequence
   * @param out constructed suffix array, will be owerwrittent
   */
  template <typename _Iterator>
  static void buildSuffixArray(
      _Iterator first,
      _Iterator last, 
      std::vector<int> *out) {
    typename std::iterator_traits<_Iterator>::difference_type length = last - first;

    out->resize(length);
    for (int i = 0; i < length; i++) {
      out->at(i) = i;
    }
    SortHelper<_Iterator> helper(first, last);
    std::sort(out->begin(), out->end(), helper);
  }
};

} // namespace suffix_array
} // namespace strings

#endif
