#include "gtest/gtest.h"
#include "naive.h"
#include "strings/utils/sequence_loader.h"
#include "utils/benchmark/benchmark.h"
#include <string>
#include "strings/testdata.h"
#include "utils/macros/foreach.h"

namespace strings {
namespace suffix_array {

TEST(NaiveSuffixArray, buildSpeed) {

  std::map<unsigned char, unsigned char> mapa;
  for (int c = 0; c < 256; c++) {
    mapa[(unsigned char) c] = (unsigned char) c;
  }

  std::string base = "../";
  std::vector<std::string> files;
  files.push_back(base + TestdataFiles::ARTIFICIAL_RANDOM);
  files.push_back(base + TestdataFiles::ARTIFICIAL_PI);
  files.push_back(base + TestdataFiles::GENOME_SHORT);
  files.push_back(base + TestdataFiles::TEXT_BIBLE);
  files.push_back(base + TestdataFiles::GENOME_ECOLI);
  files.push_back(base + TestdataFiles::ARTIFICIAL_ALPHABET_SMALL);
  files.push_back(base + TestdataFiles::ARTIFICIAL_ALPHABET_BIG);

  FOREACH(it, files) {
    std::vector<unsigned char> sequence;
    std::vector<int> result;
    sequence = utils::SequenceLoader::loadSequence(it->c_str(), mapa);
    printf("file %s (sequence size %lu)\n", it->c_str(),
        (unsigned long) sequence.size());
    BENCHMARK(1,
      NaiveSuffixArray::buildSuffixArray(sequence.begin(), sequence.end(), &result));
  }
}

} // namespace suffix_array
} // namespace string
