#ifndef H_STRINGS_SUFFIX_ARRAY_NAIVE_SORT_HELPER
#define H_STRINGS_SUFFIX_ARRAY_NAIVE_SORT_HELPER

#include <iterator>
#include "utils/preconditions/preconditions.h"
#include "utils/branch_predict/branch_predict.h"

namespace strings {
namespace suffix_array {

/**
 * Helper for sorting
 * Can compare two suffixes
 */
template <typename _Iterator>
class SortHelper {
 public:
  /**
   * @param input whole sequence
   * @param length length of the sequence
   */
  SortHelper(const _Iterator first, const _Iterator last_) {
    base = first;
    last = last_;
  }

  /**
   * Returns which of the suffixes is lexicographically smaller.
   * Note that end of sequence is less than any of the characters,
   * i.e. "x" < "xa"
   *
   * @returns true iff suffix[a] < suffix[b]
   */
  bool operator()(const int& a, const int& b) {
    int l = last - base;
    Preconditions::checkRange(a, l);
    Preconditions::checkRange(b, l);
    if (a == b) return false;
    _Iterator ita = base + a;
    _Iterator itb = base + b;

    while ((ita != last) && (itb != last) && (*ita == *itb)) {
      ++ita;
      ++itb;
    }
    if (UNLIKELY(ita == last)) return true; // suffix a is shorter
    if (UNLIKELY(itb == last)) return false; // suffix b is shorter 
    return *ita < *itb;
  }

  private:
   _Iterator base;
   _Iterator last;
};

} // namespace suffix_array
} // namespace strings

#endif
