#include "gtest/gtest.h"
#include "naive.h"
#include "strings/cyclic/duval.h"
#include "strings/utils/sequence_loader.h"
#include "utils/benchmark/benchmark.h"
#include "strings/testdata.h"
#include <string>
#include "utils/macros/foreach.h"

namespace strings {
namespace suffix_array {

TEST(NaiveSuffixArray, duvalSuffix) {

  std::map<unsigned char, unsigned char> mapa;
  for (int c = 0; c < 256; c++) {
    mapa[(unsigned char) c] = (unsigned char) c;
  }

  std::string base = "../";
  std::vector<std::string> files;

  files.push_back(base + TestdataFiles::ARTIFICIAL_RANDOM);
  files.push_back(base + TestdataFiles::ARTIFICIAL_PI);
  files.push_back(base + TestdataFiles::GENOME_SHORT);
  files.push_back(base + TestdataFiles::TEXT_BIBLE);
  files.push_back(base + TestdataFiles::GENOME_ECOLI);

  FOREACH(it, files) {
    std::vector<unsigned char> sequence;
    std::vector<int> suf_result;
    std::vector<int> duv_result;
    sequence = utils::SequenceLoader::loadSequence(it->c_str(), mapa);
    printf("file %s (sequence size %lu)\n", it->c_str(),
        (unsigned long) sequence.size());

    NaiveSuffixArray::buildSuffixArray(
        sequence.begin(), sequence.end(), &suf_result);
    cyclic::Duval<unsigned char>::minimumSuffixes(&sequence[0], sequence.size(),
        &duv_result);

    EXPECT_EQ(suf_result[0], duv_result[sequence.size()-1]);
  }
}

} // namespace suffix_array
} // namespace strings
