#ifndef H_STRINGS_TESTDATA
#define H_STRINGS_TESTDATA
/**
 * This file contains testdata filename constants
 */

#include <string>

namespace strings {

/**
 * Contains names of the test files
 */
class TestdataFiles {
 public:
  /** File with random base64 characters, size = 1M */
  static const char* ARTIFICIAL_RANDOM;

  /** File with repeated pattern "abcd....z", size = 30k */
  static const char* ARTIFICIAL_ALPHABET_SMALL;

  /** File with repeated pattern "abcd....z", size = 100k */
  static const char* ARTIFICIAL_ALPHABET_BIG;


  /** File with repeated "a", size = 30k */
  static const char* ARTIFICIAL_AAA_SMALL;

  /** File with repeated "a", size = 100k */
  static const char* ARTIFICIAL_AAA_BIG;

  /** Text file with bible, size = 4M */
  static const char* TEXT_BIBLE;

  /** Text file with world factbook, size = 2M */
  static const char* TEXT_FACTBOOK;

  /** E.coli genome sequence, size = 4.6M */
  static const char* GENOME_ECOLI;

  /** Text representation of Pi number up to 1M digits */
  static const char* ARTIFICIAL_PI;

  /** Short human genome sequence */
  static const char* GENOME_SHORT;

  /** Human genome for Y chromosome, cca 60M */
  static const char* GENOME_CHROMOSOME_Y;

  /** Source code tar archive for php, cca 93M */
  static const char* SOURCE_CODE_PHP;

  /** Apache logs, cca 16M */
  static const char* TEXT_APACHE_LOGS;
};

/**
 * Contains name of the testdata used as search patterns
 */
class PatternFiles {
 public:
  /** @var NULL-terminated array of pattern filenames*/
  static const char* SEARCH_PATTERNS[];
};


const char* TestdataFiles::ARTIFICIAL_RANDOM =
  "testdata/artificial/random.txt";

const char* TestdataFiles::ARTIFICIAL_ALPHABET_SMALL =
  "testdata/artificial/alphabet.small.txt";

const char* TestdataFiles::ARTIFICIAL_ALPHABET_BIG =
  "testdata/artificial/alphabet.txt";

const char* TestdataFiles::ARTIFICIAL_AAA_SMALL =
  "testdata/artificial/aaa.small.txt";

const char* TestdataFiles::ARTIFICIAL_AAA_BIG =
  "testdata/artificial/aaa.txt";

const char* TestdataFiles::ARTIFICIAL_PI = 
  "testdata/misc/pi.txt";

const char* TestdataFiles::TEXT_BIBLE =
  "testdata/larger/bible.txt";

const char* TestdataFiles::TEXT_FACTBOOK = 
  "testdata/larger/world192.txt";

const char* TestdataFiles::GENOME_ECOLI = 
  "testdata/larger/E.coli";

const char* TestdataFiles::GENOME_SHORT =
  "testdata/genome/chrUn_gl000211.fa";

const char* TestdataFiles::GENOME_CHROMOSOME_Y = 
  "testdata/genome/chrY.fa";

const char* TestdataFiles::SOURCE_CODE_PHP = 
  "testdata/big/php-5.3.5.tar";

const char* TestdataFiles::TEXT_APACHE_LOGS =
  "testdata/big/access.log";

const char* PatternFiles::SEARCH_PATTERNS[] = {
  "testdata/patterns/aaa.txt",
  "testdata/patterns/alphabet.txt",
  "testdata/patterns/and.txt",
  "testdata/patterns/nnn.txt",
  "testdata/patterns/php.txt",
  "testdata/patterns/php_line.txt",
  "testdata/patterns/random_genome.txt",
  "testdata/patterns/random_genome2.txt",
  "testdata/patterns/random_keyboard.txt",
  NULL
};
} // namespace strings
#endif
