#include "gtest/gtest.h"
#include "utils/benchmark/benchmark.h" 

#include "sequence_helper.h"
#include "utils/si_units/si_units.h"

namespace strings {
namespace utils {


template<typename ArrayLike>
inline void iterateElements(ArrayLike& a, int size) {
  for (int i = 0; i < size; i++) {
    a[i] = i;
  }
}

const int N = 100000000;
int data[N];

TEST(SequenceHelper, benchmarkBig) {
  SequenceHelper<int> seq(data, 10 * Mi);
  std::vector<int> vect(data, data + 10 * Mi);

  BENCHMARK(100, iterateElements(data, 10 * Mi));
  BENCHMARK(100, iterateElements(vect, 10 * Mi));
  BENCHMARK(100, iterateElements(seq, 10 * Mi));
}

TEST(SequenceHelper, benchmarkMedium) {
  SequenceHelper<int> seq(data, 100 * Ki);
  std::vector<int> vect(data, data + 100 * Ki);

  BENCHMARK(10000, iterateElements(data, 100 * Ki));
  BENCHMARK(10000, iterateElements(vect, 100 * Ki));
  BENCHMARK(10000, iterateElements(seq, 100 * Ki));
}

TEST(SequenceHelper, benchmarkSmall) {
  SequenceHelper<int> seq(data, 1 * Ki);
  std::vector<int> vect(data, data + 1 * Ki);

  BENCHMARK(1000000, iterateElements(data, 1 * Ki));
  BENCHMARK(1000000, iterateElements(vect, 1 * Ki));
  BENCHMARK(1000000, iterateElements(seq, 1 * Ki));
}

TEST(SequenceHelper, benchmarkSmallest) {
  SequenceHelper<int> seq(data, 20);
  std::vector<int> vect(data, data + 20);

  BENCHMARK(100000000, iterateElements(data, 20));
  BENCHMARK(100000000, iterateElements(vect, 20));
  BENCHMARK(100000000, iterateElements(seq, 20));
}

} // namespace sequence_helper
} // namespace strings
