#include "gtest/gtest.h"
#include "sequence_helper.h"

namespace strings {
namespace utils {

TEST(SequenceHelper, construct) {
  int data[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
  SequenceHelper<int> s(data, 11);
  EXPECT_EQ(11, s.size());
  EXPECT_EQ(-5, s[0]);
  EXPECT_EQ(-4, s[1]);
  EXPECT_EQ(5, s[10]);

  EXPECT_THROW(s[-1], std::invalid_argument);
  EXPECT_THROW(s[11], std::invalid_argument);
}

TEST(SequenceHelper, subsequence) {
  int data[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
  SequenceHelper<int> s(data, 11);
  s = s.subsequence(5, 11);
  EXPECT_EQ(6, s.size());
  EXPECT_EQ(0, s[0]);
  EXPECT_EQ(1, s[1]);
  EXPECT_EQ(5, s[5]);
  EXPECT_THROW(s[6], std::invalid_argument);
}


TEST(SequenceHelper, reverse) {
  int data[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
  SequenceHelper<int> s(data, 11);
  s = s.reversed();
  EXPECT_EQ(11, s.size());
  EXPECT_EQ(5, s[0]);
  EXPECT_EQ(4, s[1]);
  EXPECT_EQ(3, s[2]);
  EXPECT_EQ(-5, s[10]);

  EXPECT_THROW(s[-1], std::invalid_argument);
  EXPECT_THROW(s[11], std::invalid_argument);

}

TEST(SequenceHelper, reverseAndSubsequence) {
  int data[11] = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5};
  SequenceHelper<int> s(data, 11);

  EXPECT_EQ(-5, s[0]);

  s = s.reversed();
  // 5 4 .... -4 -5

  EXPECT_EQ(5, s[0]);

  s = s.subsequence(2, 10);
  // 3 2 .... -4
  EXPECT_EQ(8, s.size());
  EXPECT_EQ(3, s[0]);
  EXPECT_EQ(-4, s[7]);

  s = s.reversed();
  // -4 -3 .... 2 3
  EXPECT_EQ(8, s.size());
  EXPECT_EQ(-4, s[0]);
  EXPECT_EQ(3, s[7]);

  s = s.subsequence(2, 6);
  // -2 -1 0 1
  EXPECT_EQ(4, s.size());
  EXPECT_EQ(-2, s[0]);
  EXPECT_EQ(1, s[3]);

  s = s.reversed();
  // 1 0 -1 -2
  EXPECT_EQ(4, s.size());
  EXPECT_EQ(1, s[0]);
  EXPECT_EQ(-2, s[3]);

  s = s.subsequence(1, 1);
  EXPECT_EQ(0, s.size());
}

} // namespace sequence_helper
} // namespace strings
