#ifndef H_UTILS_BENCHMARK_COLOR
#define H_UTILS_BENCHMARK_COLOR
/**
 * @file Contains colorPrintf function which support color printing
 */
#include <stdarg.h>

#ifdef _WIN32
  #include <windows.h>
  #include <wincon.h>
  #include <io.h>
  namespace color {
    /**
     * Available colors for color printing
     */
    enum Color {
      BLUE = FOREGROUND_BLUE,
      PINK = FOREGROUND_BLUE | FOREGROUND_RED,
      CYAN = FOREGROUND_BLUE | FOREGROUND_GREEN,
    };

    /**
     * Prints message in color.
     * The usage is same as for the printf.
     *
     * @param color in which text should be displayed
     * @param fmt format string
     */
    void colorPrintf(Color color, const char* fmt, ...) {
      // This function is copied from GTEST!

      va_list args;
      va_start(args, fmt);

      const HANDLE stdout_handle = GetStdHandle(STD_OUTPUT_HANDLE);

      // Gets the current text color.
      CONSOLE_SCREEN_BUFFER_INFO buffer_info;
      GetConsoleScreenBufferInfo(stdout_handle, &buffer_info);
      const WORD old_color_attrs = buffer_info.wAttributes;

      // We need to flush the stream buffers into the console before each
      // SetConsoleTextAttribute call lest it affect the text that is already
      // printed but has not yet reached the console.
      fflush(stdout);
      SetConsoleTextAttribute(stdout_handle,
                              color | FOREGROUND_INTENSITY);
      vprintf(fmt, args);

      fflush(stdout);
      // Restores the text color.
      SetConsoleTextAttribute(stdout_handle, old_color_attrs);
    }

  }
#else
  namespace color {
    /**
     * Available colors for color printing
     */
    enum Color {
      BLUE = 34,
      PINK = 35,
      CYAN = 36,
    };

    /**
     * Prints message in color.
     * The usage is same as for the printf.
     *
     * @param color in which text should be displayed
     * @param fmt format string
     */
    void colorPrintf(Color color, const char* fmt, ...) {
      va_list args;
      va_start(args, fmt);
      printf("%c[1;%dm", 27, (int) color);
      vprintf(fmt, args);
      printf("%c[0m", 27);
    }

  }
#endif

#endif
