#ifndef H_UTILS_MEMORY_USAGE_MEMORY_USAGE
#define H_UTILS_MEMORY_USAGE_MEMORY_USAGE
/**
 * @file Contains utilities helping to determine size of
 *  memory currently used by the program.
 */
#include <malloc.h>

namespace utils {
namespace memory_usage {

/**
 * Return the current allocated memory.
 *
 * @warning this only counts memory allocated by
 *  malloc() (includes all STL structures and new() operator)
 * @warning does not include shared objects
 * @warning real memory usage may be bigger, because malloc()
 *   can keep some pieces of memory for later reuse
 *
 * @returns currently used memory in kilobytes
 */
int getUsedMemoryKb() {
  return mallinfo().uordblks / 1024;
}

} // namespace memory_usage
} // namespace utils

#endif
