#include <limits.h>
#include <cmath>

Rand::Rand(unsigned int seed) {
  my_seed = seed;
}

const unsigned int RandMax = (1u<<31) - 2;


unsigned int Rand::rand() {
  my_seed = my_seed * 1103515245 + 12345;
  return my_seed % (1 + RandMax);
}

// random number from interval <min, max)
int Rand::rand(int min, int max) {
  Preconditions::check(min < max);
  unsigned int l = max - min;
  unsigned int top = l * (RandMax / l);
  unsigned int res;
  do {
    res = rand();
  } while (res >= top);
  return (res % l) + min;
}

// random number with exponential distribution
// (i.e. log(x) is uniform distribution on some interval)
// Warning - it is not platform-safe
// TODO(ppershing): make only one rand() call and
// fix results instead
int Rand::exprand(unsigned int min, unsigned int max) {
  Preconditions::check(min>0, "min should be positive");
  Preconditions::check(min<max, "min should be less than max");

  // skip the randomness
  if (min + 1 == max) {
    return min;
  }

  unsigned int result;
  do {
    double e = rand() / (double) UINT_MAX ;
    e = e * (log(max) - log(min)) + log(min);
    result = (unsigned int) exp(e);
  // solve improbable case of rounding error
  // at interval ends
  } while (!(result >= min && result < max));
  return result;
}

double Rand::randdouble(double min, double max) {
  Preconditions::check(min < max, "min should be less than max");

  double e = (double) rand() / (double) UINT_MAX;

  return e * (max - min) + min;
}

double Rand::expranddouble(double min, double max) {
  Preconditions::check(min > 0);
  Preconditions::check(min < max);
  return exp(randdouble(log(min), log(max)));
}
