#ifndef H_STATIC_ASSERT
#define H_STATIC_ASSERT
/**
 * @file Contains macro with which you can assert in compile-time
 */

namespace utils {
namespace static_assert_ {

/**
 * Helper structure for static assertions
 */
template <bool x> struct STATIC_ASSERTION_FAILURE;

/**
 * specialization of helper structure for valid expressions
 */
template <> struct STATIC_ASSERTION_FAILURE<true> {
  enum { value = 1 };
};

/**
 * Helper structure for static assertions
 */
template<int x> struct static_assert_test{};

// We need two definitions because otherwise JOIN(x, __LINE__) will
// end up as  "x__LINE__"
#define __JOIN(X, Y) __JOIN2(X, Y)
#define __JOIN2(X, Y) X##Y

/**
 * Static assertion.
 * You can assert on any compile-time expressions.
 * Note: if you try to pass non-compile-time expression, compilation
 * will be aborted
 *
 * @param B boolean expression
 * @param MSG (ignored) message - it is for compatibility with
 *   C++0x static_assert
 */
#define STATIC_ASSERT( B, MSG ) \
  typedef ::utils::static_assert_::static_assert_test<\
      sizeof(::utils::static_assert_::STATIC_ASSERTION_FAILURE<(bool) (B)>)> \
      __JOIN(static_assert_on_line_, __LINE__);

} // namespace static_assert
} // namespace utils
#endif
